# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import (
    account_collection_flow_list_params,
    account_collection_flow_create_params,
    account_collection_flow_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.account_collection_flow import AccountCollectionFlow

__all__ = ["AccountCollectionFlows", "AsyncAccountCollectionFlows"]


class AccountCollectionFlows(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountCollectionFlowsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AccountCollectionFlowsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountCollectionFlowsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AccountCollectionFlowsWithStreamingResponse(self)

    def create(
        self,
        *,
        counterparty_id: str,
        payment_types: List[str],
        receiving_countries: List[
            Literal[
                "USA",
                "AUS",
                "BEL",
                "CAN",
                "CHL",
                "CHN",
                "COL",
                "FRA",
                "DEU",
                "HKG",
                "IND",
                "IRL",
                "ITA",
                "MEX",
                "NLD",
                "PER",
                "ESP",
                "GBR",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AccountCollectionFlow:
        """
        create account_collection_flow

        Args:
          counterparty_id: Required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/account_collection_flows",
            body=maybe_transform(
                {
                    "counterparty_id": counterparty_id,
                    "payment_types": payment_types,
                    "receiving_countries": receiving_countries,
                },
                account_collection_flow_create_params.AccountCollectionFlowCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AccountCollectionFlow,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountCollectionFlow:
        """
        get account_collection_flow

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/account_collection_flows/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountCollectionFlow,
        )

    def update(
        self,
        id: str,
        *,
        status: Literal["cancelled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AccountCollectionFlow:
        """update account_collection_flow

        Args:
          status: Required.

        The updated status of the account collection flow. Can only be used to
              mark a flow as `cancelled`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/account_collection_flows/{id}",
            body=maybe_transform(
                {"status": status}, account_collection_flow_update_params.AccountCollectionFlowUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AccountCollectionFlow,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        client_token: str | NotGiven = NOT_GIVEN,
        counterparty_id: str | NotGiven = NOT_GIVEN,
        external_account_id: str | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[AccountCollectionFlow]:
        """
        list account_collection_flows

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/account_collection_flows",
            page=SyncPage[AccountCollectionFlow],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "client_token": client_token,
                        "counterparty_id": counterparty_id,
                        "external_account_id": external_account_id,
                        "per_page": per_page,
                        "status": status,
                    },
                    account_collection_flow_list_params.AccountCollectionFlowListParams,
                ),
            ),
            model=AccountCollectionFlow,
        )


class AsyncAccountCollectionFlows(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountCollectionFlowsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountCollectionFlowsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountCollectionFlowsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncAccountCollectionFlowsWithStreamingResponse(self)

    async def create(
        self,
        *,
        counterparty_id: str,
        payment_types: List[str],
        receiving_countries: List[
            Literal[
                "USA",
                "AUS",
                "BEL",
                "CAN",
                "CHL",
                "CHN",
                "COL",
                "FRA",
                "DEU",
                "HKG",
                "IND",
                "IRL",
                "ITA",
                "MEX",
                "NLD",
                "PER",
                "ESP",
                "GBR",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AccountCollectionFlow:
        """
        create account_collection_flow

        Args:
          counterparty_id: Required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/account_collection_flows",
            body=await async_maybe_transform(
                {
                    "counterparty_id": counterparty_id,
                    "payment_types": payment_types,
                    "receiving_countries": receiving_countries,
                },
                account_collection_flow_create_params.AccountCollectionFlowCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AccountCollectionFlow,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountCollectionFlow:
        """
        get account_collection_flow

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/account_collection_flows/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountCollectionFlow,
        )

    async def update(
        self,
        id: str,
        *,
        status: Literal["cancelled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> AccountCollectionFlow:
        """update account_collection_flow

        Args:
          status: Required.

        The updated status of the account collection flow. Can only be used to
              mark a flow as `cancelled`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/account_collection_flows/{id}",
            body=await async_maybe_transform(
                {"status": status}, account_collection_flow_update_params.AccountCollectionFlowUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AccountCollectionFlow,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        client_token: str | NotGiven = NOT_GIVEN,
        counterparty_id: str | NotGiven = NOT_GIVEN,
        external_account_id: str | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        status: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[AccountCollectionFlow, AsyncPage[AccountCollectionFlow]]:
        """
        list account_collection_flows

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/account_collection_flows",
            page=AsyncPage[AccountCollectionFlow],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "client_token": client_token,
                        "counterparty_id": counterparty_id,
                        "external_account_id": external_account_id,
                        "per_page": per_page,
                        "status": status,
                    },
                    account_collection_flow_list_params.AccountCollectionFlowListParams,
                ),
            ),
            model=AccountCollectionFlow,
        )


class AccountCollectionFlowsWithRawResponse:
    def __init__(self, account_collection_flows: AccountCollectionFlows) -> None:
        self._account_collection_flows = account_collection_flows

        self.create = _legacy_response.to_raw_response_wrapper(
            account_collection_flows.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            account_collection_flows.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            account_collection_flows.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            account_collection_flows.list,
        )


class AsyncAccountCollectionFlowsWithRawResponse:
    def __init__(self, account_collection_flows: AsyncAccountCollectionFlows) -> None:
        self._account_collection_flows = account_collection_flows

        self.create = _legacy_response.async_to_raw_response_wrapper(
            account_collection_flows.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            account_collection_flows.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            account_collection_flows.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            account_collection_flows.list,
        )


class AccountCollectionFlowsWithStreamingResponse:
    def __init__(self, account_collection_flows: AccountCollectionFlows) -> None:
        self._account_collection_flows = account_collection_flows

        self.create = to_streamed_response_wrapper(
            account_collection_flows.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            account_collection_flows.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            account_collection_flows.update,
        )
        self.list = to_streamed_response_wrapper(
            account_collection_flows.list,
        )


class AsyncAccountCollectionFlowsWithStreamingResponse:
    def __init__(self, account_collection_flows: AsyncAccountCollectionFlows) -> None:
        self._account_collection_flows = account_collection_flows

        self.create = async_to_streamed_response_wrapper(
            account_collection_flows.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            account_collection_flows.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            account_collection_flows.update,
        )
        self.list = async_to_streamed_response_wrapper(
            account_collection_flows.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .line_item_list_params import LineItemListParams as LineItemListParams
from .transaction_line_item import TransactionLineItem as TransactionLineItem
from .line_item_create_params import LineItemCreateParams as LineItemCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .bank_settings_param import BankSettingsParam
from .external_account_type import ExternalAccountType
from .shared.transaction_direction import TransactionDirection
from .wealth_and_employment_details_param import WealthAndEmploymentDetailsParam
from .shared_params.legal_entity_compliance_detail import LegalEntityComplianceDetail
from .shared_params.legal_entity_industry_classification import LegalEntityIndustryClassification

__all__ = [
    "CounterpartyCreateParams",
    "Accounting",
    "Accounts",
    "Account",
    "AccountsAccountDetails",
    "AccountAccountDetail",
    "AccountsContactDetails",
    "AccountContactDetail",
    "AccountsLedgerAccount",
    "AccountLedgerAccount",
    "AccountsPartyAddress",
    "AccountPartyAddress",
    "AccountsRoutingDetails",
    "AccountRoutingDetail",
    "LegalEntity",
    "LegalEntityAddresses",
    "LegalEntityAddress",
    "LegalEntityIdentifications",
    "LegalEntityIdentification",
    "LegalEntityLegalEntityAssociations",
    "LegalEntityLegalEntityAssociation",
    "LegalEntityLegalEntityAssociationsChildLegalEntity",
    "LegalEntityLegalEntityAssociationChildLegalEntity",
    "LegalEntityLegalEntityAssociationsChildLegalEntityAddresses",
    "LegalEntityLegalEntityAssociationChildLegalEntityAddress",
    "LegalEntityLegalEntityAssociationsChildLegalEntityIdentifications",
    "LegalEntityLegalEntityAssociationChildLegalEntityIdentification",
    "LegalEntityLegalEntityAssociationsChildLegalEntityPhoneNumbers",
    "LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber",
    "LegalEntityPhoneNumbers",
    "LegalEntityPhoneNumber",
]


class CounterpartyCreateParams(TypedDict, total=False):
    name: Required[Optional[str]]
    """A human friendly name for this counterparty."""

    accounting: Accounting

    accounts: Iterable[Account]
    """The accounts for this counterparty."""

    email: Optional[str]
    """The counterparty's email."""

    ledger_type: Literal["customer", "vendor"]
    """An optional type to auto-sync the counterparty to your ledger.

    Either `customer` or `vendor`.
    """

    legal_entity: LegalEntity

    legal_entity_id: Optional[str]
    """The id of the legal entity."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    send_remittance_advice: bool
    """
    Send an email to the counterparty whenever an associated payment order is sent
    to the bank.
    """

    taxpayer_identifier: str
    """Either a valid SSN or EIN."""

    verification_status: Literal["denied", "needs_approval", "unverified", "verified"]
    """The verification status of the counterparty."""


class Accounting(TypedDict, total=False):
    type: Literal["customer", "vendor"]
    """An optional type to auto-sync the counterparty to your ledger.

    Either `customer` or `vendor`.
    """


class AccountAccountDetail(TypedDict, total=False):
    account_number: Required[str]

    account_number_type: Literal[
        "au_number",
        "base_address",
        "clabe",
        "ethereum_address",
        "hk_number",
        "iban",
        "id_number",
        "nz_number",
        "other",
        "pan",
        "polygon_address",
        "sg_number",
        "solana_address",
        "wallet_address",
    ]


AccountsAccountDetails = AccountAccountDetail
"""This type is deprecated and will be removed in a future release.

Please use AccountAccountDetail instead.
"""


class AccountContactDetail(TypedDict, total=False):
    contact_identifier: str

    contact_identifier_type: Literal["email", "phone_number", "website"]


AccountsContactDetails = AccountContactDetail
"""This type is deprecated and will be removed in a future release.

Please use AccountContactDetail instead.
"""


class AccountLedgerAccount(TypedDict, total=False):
    currency: Required[str]
    """The currency of the ledger account."""

    ledger_id: Required[str]
    """The id of the ledger that this account belongs to."""

    name: Required[str]
    """The name of the ledger account."""

    normal_balance: Required[TransactionDirection]
    """The normal balance of the ledger account."""

    currency_exponent: Optional[int]
    """The currency exponent of the ledger account."""

    description: Optional[str]
    """The description of the ledger account."""

    ledger_account_category_ids: List[str]
    """
    The array of ledger account category ids that this ledger account should be a
    child of.
    """

    ledgerable_id: str
    """
    If the ledger account links to another object in Modern Treasury, the id will be
    populated here, otherwise null.
    """

    ledgerable_type: Literal["counterparty", "external_account", "internal_account", "virtual_account"]
    """
    If the ledger account links to another object in Modern Treasury, the type will
    be populated here, otherwise null. The value is one of internal_account or
    external_account.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """


AccountsLedgerAccount = AccountLedgerAccount
"""This type is deprecated and will be removed in a future release.

Please use AccountLedgerAccount instead.
"""


class AccountPartyAddress(TypedDict, total=False):
    country: Optional[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Optional[str]

    line2: Optional[str]

    locality: Optional[str]
    """Locality or City."""

    postal_code: Optional[str]
    """The postal code of the address."""

    region: Optional[str]
    """Region or State."""


AccountsPartyAddress = AccountPartyAddress
"""This type is deprecated and will be removed in a future release.

Please use AccountPartyAddress instead.
"""


class AccountRoutingDetail(TypedDict, total=False):
    routing_number: Required[str]

    routing_number_type: Required[
        Literal[
            "aba",
            "au_bsb",
            "br_codigo",
            "ca_cpa",
            "chips",
            "cnaps",
            "dk_interbank_clearing_code",
            "gb_sort_code",
            "hk_interbank_clearing_code",
            "hu_interbank_clearing_code",
            "id_sknbi_code",
            "in_ifsc",
            "jp_zengin_code",
            "my_branch_code",
            "mx_bank_identifier",
            "nz_national_clearing_code",
            "pl_national_clearing_code",
            "se_bankgiro_clearing_code",
            "sg_interbank_clearing_code",
            "swift",
            "za_national_clearing_code",
        ]
    ]

    payment_type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "wire",
        "zengin",
    ]


AccountsRoutingDetails = AccountRoutingDetail
"""This type is deprecated and will be removed in a future release.

Please use AccountRoutingDetail instead.
"""


class Account(TypedDict, total=False):
    account_details: Iterable[AccountAccountDetail]

    account_type: ExternalAccountType
    """Can be `checking`, `savings` or `other`."""

    contact_details: Iterable[AccountContactDetail]

    ledger_account: AccountLedgerAccount
    """Specifies a ledger account object that will be created with the external
    account.

    The resulting ledger account is linked to the external account for
    auto-ledgering Payment objects. See
    https://docs.moderntreasury.com/docs/linking-to-other-modern-treasury-objects
    for more details.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str]
    """A nickname for the external account.

    This is only for internal usage and won't affect any payments
    """

    party_address: AccountPartyAddress
    """Required if receiving wire payments."""

    party_identifier: str

    party_name: str
    """
    If this value isn't provided, it will be inherited from the counterparty's name.
    """

    party_type: Optional[Literal["business", "individual"]]
    """Either `individual` or `business`."""

    plaid_processor_token: str
    """
    If you've enabled the Modern Treasury + Plaid integration in your Plaid account,
    you can pass the processor token in this field.
    """

    routing_details: Iterable[AccountRoutingDetail]


Accounts = Account
"""This type is deprecated and will be removed in a future release.

Please use Account instead.
"""


class LegalEntityAddress(TypedDict, total=False):
    country: Required[Optional[str]]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[Optional[str]]

    locality: Required[Optional[str]]
    """Locality or City."""

    postal_code: Required[Optional[str]]
    """The postal code of the address."""

    region: Required[Optional[str]]
    """Region or State."""

    address_types: List[Literal["business", "mailing", "other", "po_box", "residential"]]
    """The types of this address."""

    line2: Optional[str]


LegalEntityAddresses = LegalEntityAddress
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityAddress instead.
"""


class LegalEntityIdentification(TypedDict, total=False):
    id_number: Required[str]
    """The ID number of identification document."""

    id_type: Required[
        Literal[
            "ar_cuil",
            "ar_cuit",
            "br_cnpj",
            "br_cpf",
            "cl_run",
            "cl_rut",
            "co_cedulas",
            "co_nit",
            "drivers_license",
            "hn_id",
            "hn_rtn",
            "in_lei",
            "kr_brn",
            "kr_crn",
            "kr_rrn",
            "passport",
            "sa_tin",
            "sa_vat",
            "us_ein",
            "us_itin",
            "us_ssn",
            "vn_tin",
        ]
    ]
    """The type of ID number."""

    expiration_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    The date when the Identification is no longer considered valid by the issuing
    authority.
    """

    issuing_country: Optional[str]
    """
    The ISO 3166-1 alpha-2 country code of the country that issued the
    identification
    """

    issuing_region: Optional[str]
    """The region in which the identifcation was issued."""


LegalEntityIdentifications = LegalEntityIdentification
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityIdentification instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntityAddress(TypedDict, total=False):
    country: Required[Optional[str]]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Required[Optional[str]]

    locality: Required[Optional[str]]
    """Locality or City."""

    postal_code: Required[Optional[str]]
    """The postal code of the address."""

    region: Required[Optional[str]]
    """Region or State."""

    address_types: List[Literal["business", "mailing", "other", "po_box", "residential"]]
    """The types of this address."""

    line2: Optional[str]


LegalEntityLegalEntityAssociationsChildLegalEntityAddresses = LegalEntityLegalEntityAssociationChildLegalEntityAddress
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntityAddress instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntityIdentification(TypedDict, total=False):
    id_number: Required[str]
    """The ID number of identification document."""

    id_type: Required[
        Literal[
            "ar_cuil",
            "ar_cuit",
            "br_cnpj",
            "br_cpf",
            "cl_run",
            "cl_rut",
            "co_cedulas",
            "co_nit",
            "drivers_license",
            "hn_id",
            "hn_rtn",
            "in_lei",
            "kr_brn",
            "kr_crn",
            "kr_rrn",
            "passport",
            "sa_tin",
            "sa_vat",
            "us_ein",
            "us_itin",
            "us_ssn",
            "vn_tin",
        ]
    ]
    """The type of ID number."""

    expiration_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    The date when the Identification is no longer considered valid by the issuing
    authority.
    """

    issuing_country: Optional[str]
    """
    The ISO 3166-1 alpha-2 country code of the country that issued the
    identification
    """

    issuing_region: Optional[str]
    """The region in which the identifcation was issued."""


LegalEntityLegalEntityAssociationsChildLegalEntityIdentifications = (
    LegalEntityLegalEntityAssociationChildLegalEntityIdentification
)
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntityIdentification instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber(TypedDict, total=False):
    phone_number: str


LegalEntityLegalEntityAssociationsChildLegalEntityPhoneNumbers = (
    LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber
)
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber instead.
"""


class LegalEntityLegalEntityAssociationChildLegalEntity(TypedDict, total=False):
    addresses: Iterable[LegalEntityLegalEntityAssociationChildLegalEntityAddress]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettingsParam]

    business_name: Optional[str]
    """The business's legal business name."""

    citizenship_country: Optional[str]
    """The country of citizenship for an individual."""

    compliance_details: Optional[LegalEntityComplianceDetail]

    date_formed: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """An individual's date of birth (YYYY-MM-DD)."""

    doing_business_as_names: List[str]

    email: Optional[str]
    """The entity's primary email."""

    first_name: Optional[str]
    """An individual's first name."""

    identifications: Iterable[LegalEntityLegalEntityAssociationChildLegalEntityIdentification]
    """A list of identifications for the legal entity."""

    industry_classifications: Iterable[LegalEntityIndustryClassification]
    """A list of industry classifications for the legal entity."""

    last_name: Optional[str]
    """An individual's last name."""

    legal_entity_type: Literal["business", "individual"]
    """The type of legal entity."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ]
    """The business's legal structure."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str]
    """An individual's middle name."""

    phone_numbers: Iterable[LegalEntityLegalEntityAssociationChildLegalEntityPhoneNumber]

    politically_exposed_person: Optional[bool]
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str]
    """An individual's preferred name."""

    prefix: Optional[str]
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]]
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str]
    """An individual's suffix."""

    wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam]

    website: Optional[str]
    """The entity's primary website URL."""


LegalEntityLegalEntityAssociationsChildLegalEntity = LegalEntityLegalEntityAssociationChildLegalEntity
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociationChildLegalEntity instead.
"""


class LegalEntityLegalEntityAssociation(TypedDict, total=False):
    relationship_types: Required[List[Literal["beneficial_owner", "control_person"]]]

    child_legal_entity: LegalEntityLegalEntityAssociationChildLegalEntity
    """The child legal entity."""

    child_legal_entity_id: str
    """The ID of the child legal entity."""

    ownership_percentage: Optional[int]
    """The child entity's ownership percentage iff they are a beneficial owner."""

    title: Optional[str]
    """The job title of the child entity at the parent entity."""


LegalEntityLegalEntityAssociations = LegalEntityLegalEntityAssociation
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityLegalEntityAssociation instead.
"""


class LegalEntityPhoneNumber(TypedDict, total=False):
    phone_number: str


LegalEntityPhoneNumbers = LegalEntityPhoneNumber
"""This type is deprecated and will be removed in a future release.

Please use LegalEntityPhoneNumber instead.
"""


class LegalEntity(TypedDict, total=False):
    legal_entity_type: Required[Literal["business", "individual"]]
    """The type of legal entity."""

    addresses: Iterable[LegalEntityAddress]
    """A list of addresses for the entity."""

    bank_settings: Optional[BankSettingsParam]

    business_name: Optional[str]
    """The business's legal business name."""

    citizenship_country: Optional[str]
    """The country of citizenship for an individual."""

    compliance_details: Optional[LegalEntityComplianceDetail]

    date_formed: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """A business's formation date (YYYY-MM-DD)."""

    date_of_birth: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """An individual's date of birth (YYYY-MM-DD)."""

    doing_business_as_names: List[str]

    email: Optional[str]
    """The entity's primary email."""

    first_name: Optional[str]
    """An individual's first name."""

    identifications: Iterable[LegalEntityIdentification]
    """A list of identifications for the legal entity."""

    industry_classifications: Iterable[LegalEntityIndustryClassification]
    """A list of industry classifications for the legal entity."""

    last_name: Optional[str]
    """An individual's last name."""

    legal_entity_associations: Optional[Iterable[LegalEntityLegalEntityAssociation]]
    """The legal entity associations and its child legal entities."""

    legal_structure: Optional[
        Literal["corporation", "llc", "non_profit", "partnership", "sole_proprietorship", "trust"]
    ]
    """The business's legal structure."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    middle_name: Optional[str]
    """An individual's middle name."""

    phone_numbers: Iterable[LegalEntityPhoneNumber]

    politically_exposed_person: Optional[bool]
    """Whether the individual is a politically exposed person."""

    preferred_name: Optional[str]
    """An individual's preferred name."""

    prefix: Optional[str]
    """An individual's prefix."""

    risk_rating: Optional[Literal["low", "medium", "high"]]
    """The risk rating of the legal entity. One of low, medium, high."""

    suffix: Optional[str]
    """An individual's suffix."""

    wealth_and_employment_details: Optional[WealthAndEmploymentDetailsParam]

    website: Optional[str]
    """The entity's primary website URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import legal_entity_association_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .._base_client import make_request_options
from ..types.legal_entity_association import LegalEntityAssociation

__all__ = ["LegalEntityAssociations", "AsyncLegalEntityAssociations"]


class LegalEntityAssociations(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LegalEntityAssociationsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LegalEntityAssociationsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LegalEntityAssociationsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LegalEntityAssociationsWithStreamingResponse(self)

    def create(
        self,
        *,
        parent_legal_entity_id: str,
        relationship_types: List[Literal["beneficial_owner", "control_person"]],
        child_legal_entity: legal_entity_association_create_params.ChildLegalEntity | NotGiven = NOT_GIVEN,
        child_legal_entity_id: str | NotGiven = NOT_GIVEN,
        ownership_percentage: Optional[int] | NotGiven = NOT_GIVEN,
        title: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LegalEntityAssociation:
        """
        create legal_entity_association

        Args:
          parent_legal_entity_id: The ID of the parent legal entity. This must be a business or joint legal
              entity.

          child_legal_entity: The child legal entity.

          child_legal_entity_id: The ID of the child legal entity.

          ownership_percentage: The child entity's ownership percentage iff they are a beneficial owner.

          title: The job title of the child entity at the parent entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/legal_entity_associations",
            body=maybe_transform(
                {
                    "parent_legal_entity_id": parent_legal_entity_id,
                    "relationship_types": relationship_types,
                    "child_legal_entity": child_legal_entity,
                    "child_legal_entity_id": child_legal_entity_id,
                    "ownership_percentage": ownership_percentage,
                    "title": title,
                },
                legal_entity_association_create_params.LegalEntityAssociationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LegalEntityAssociation,
        )


class AsyncLegalEntityAssociations(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLegalEntityAssociationsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLegalEntityAssociationsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLegalEntityAssociationsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLegalEntityAssociationsWithStreamingResponse(self)

    async def create(
        self,
        *,
        parent_legal_entity_id: str,
        relationship_types: List[Literal["beneficial_owner", "control_person"]],
        child_legal_entity: legal_entity_association_create_params.ChildLegalEntity | NotGiven = NOT_GIVEN,
        child_legal_entity_id: str | NotGiven = NOT_GIVEN,
        ownership_percentage: Optional[int] | NotGiven = NOT_GIVEN,
        title: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LegalEntityAssociation:
        """
        create legal_entity_association

        Args:
          parent_legal_entity_id: The ID of the parent legal entity. This must be a business or joint legal
              entity.

          child_legal_entity: The child legal entity.

          child_legal_entity_id: The ID of the child legal entity.

          ownership_percentage: The child entity's ownership percentage iff they are a beneficial owner.

          title: The job title of the child entity at the parent entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/legal_entity_associations",
            body=await async_maybe_transform(
                {
                    "parent_legal_entity_id": parent_legal_entity_id,
                    "relationship_types": relationship_types,
                    "child_legal_entity": child_legal_entity,
                    "child_legal_entity_id": child_legal_entity_id,
                    "ownership_percentage": ownership_percentage,
                    "title": title,
                },
                legal_entity_association_create_params.LegalEntityAssociationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LegalEntityAssociation,
        )


class LegalEntityAssociationsWithRawResponse:
    def __init__(self, legal_entity_associations: LegalEntityAssociations) -> None:
        self._legal_entity_associations = legal_entity_associations

        self.create = _legacy_response.to_raw_response_wrapper(
            legal_entity_associations.create,
        )


class AsyncLegalEntityAssociationsWithRawResponse:
    def __init__(self, legal_entity_associations: AsyncLegalEntityAssociations) -> None:
        self._legal_entity_associations = legal_entity_associations

        self.create = _legacy_response.async_to_raw_response_wrapper(
            legal_entity_associations.create,
        )


class LegalEntityAssociationsWithStreamingResponse:
    def __init__(self, legal_entity_associations: LegalEntityAssociations) -> None:
        self._legal_entity_associations = legal_entity_associations

        self.create = to_streamed_response_wrapper(
            legal_entity_associations.create,
        )


class AsyncLegalEntityAssociationsWithStreamingResponse:
    def __init__(self, legal_entity_associations: AsyncLegalEntityAssociations) -> None:
        self._legal_entity_associations = legal_entity_associations

        self.create = async_to_streamed_response_wrapper(
            legal_entity_associations.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ExternalAccountType"]

ExternalAccountType: TypeAlias = Literal[
    "base_wallet",
    "cash",
    "checking",
    "crypto_wallet",
    "ethereum_wallet",
    "general_ledger",
    "loan",
    "non_resident",
    "other",
    "overdraft",
    "polygon_wallet",
    "savings",
    "solana_wallet",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["LedgerEntryUpdateParams"]


class LedgerEntryUpdateParams(TypedDict, total=False):
    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import date
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransactionListParams"]


class TransactionListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    as_of_date_end: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    Filters transactions with an `as_of_date` starting on or before the specified
    date (YYYY-MM-DD).
    """

    as_of_date_start: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    Filters transactions with an `as_of_date` starting on or after the specified
    date (YYYY-MM-DD).
    """

    counterparty_id: str

    description: str
    """Filters for transactions including the queried string in the description."""

    direction: str

    internal_account_id: str
    """
    Specify `internal_account_id` if you wish to see transactions to/from a specific
    account.
    """

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    payment_type: str

    per_page: int

    posted: bool
    """Either `true` or `false`."""

    transactable_type: str

    vendor_id: str
    """
    Filters for transactions including the queried vendor id (an identifier given to
    transactions by the bank).
    """

    virtual_account_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import date, datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "LedgerAccountListParams",
    "AvailableBalanceAmount",
    "Balances",
    "PendingBalanceAmount",
    "PostedBalanceAmount",
]


class LedgerAccountListParams(TypedDict, total=False):
    id: List[str]
    """
    If you have specific IDs to retrieve in bulk, you can pass them as query
    parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.
    """

    after_cursor: Optional[str]

    available_balance_amount: AvailableBalanceAmount
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), `eq` (=), or `not_eq` (!=) to
    filter by balance amount.
    """

    balances: Balances
    """
    Use `balances[effective_at_lower_bound]` and
    `balances[effective_at_upper_bound]` to get the balances change between the two
    timestamps. The lower bound is inclusive while the upper bound is exclusive of
    the provided timestamps. If no value is supplied the balances will be retrieved
    not including that bound.
    """

    created_at: Dict[str, Union[str, datetime]]
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
    created at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
    created_at%5Bgt%5D=2000-01-01T12:00:00Z.
    """

    currency: str

    external_id: str

    ledger_account_category_id: str

    ledger_id: str

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    name: List[str]
    """
    If you have specific names to retrieve in bulk, you can pass them as query
    parameters delimited with `name[]=`, for example `?name[]=123&name[]=abc`.
    """

    pending_balance_amount: PendingBalanceAmount
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), `eq` (=), or `not_eq` (!=) to
    filter by balance amount.
    """

    per_page: int

    posted_balance_amount: PostedBalanceAmount
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), `eq` (=), or `not_eq` (!=) to
    filter by balance amount.
    """

    updated_at: Dict[str, Union[str, datetime]]
    """
    Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
    updated at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
    updated_at%5Bgt%5D=2000-01-01T12:00:00Z.
    """


class AvailableBalanceAmount(TypedDict, total=False):
    eq: int

    gt: int

    gte: int

    lt: int

    lte: int

    not_eq: int


class Balances(TypedDict, total=False):
    as_of_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    effective_at_lower_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    effective_at_upper_bound: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]


class PendingBalanceAmount(TypedDict, total=False):
    eq: int

    gt: int

    gte: int

    lt: int

    lte: int

    not_eq: int


class PostedBalanceAmount(TypedDict, total=False):
    eq: int

    gt: int

    gte: int

    lt: int

    lte: int

    not_eq: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import BulkRequest
from modern_treasury._utils import parse_date, parse_datetime
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBulkRequests:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        bulk_request = client.bulk_requests.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                }
            ],
        )
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        bulk_request = client.bulk_requests.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                    "accounting": {
                        "account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "class_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "accounting_category_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "accounting_ledger_class_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "charge_bearer": "shared",
                    "currency": "AED",
                    "description": "description",
                    "effective_date": parse_date("2019-12-27"),
                    "expires_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "fallback_type": "ach",
                    "foreign_exchange_contract": "foreign_exchange_contract",
                    "foreign_exchange_indicator": "fixed_to_variable",
                    "ledger_transaction": {
                        "ledger_entries": [
                            {
                                "amount": 0,
                                "direction": "credit",
                                "ledger_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "available_balance_amount": {"foo": 0},
                                "lock_version": 0,
                                "metadata": {
                                    "key": "value",
                                    "foo": "bar",
                                    "modern": "treasury",
                                },
                                "pending_balance_amount": {"foo": 0},
                                "posted_balance_amount": {"foo": 0},
                                "show_resulting_ledger_account_balances": True,
                            }
                        ],
                        "description": "description",
                        "effective_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "effective_date": parse_date("2019-12-27"),
                        "external_id": "external_id",
                        "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "ledgerable_type": "expected_payment",
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                        "status": "archived",
                    },
                    "ledger_transaction_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "line_items": [
                        {
                            "amount": 0,
                            "accounting_category_id": "accounting_category_id",
                            "description": "description",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                        }
                    ],
                    "metadata": {
                        "key": "value",
                        "foo": "bar",
                        "modern": "treasury",
                    },
                    "nsf_protected": True,
                    "originating_party_name": "originating_party_name",
                    "priority": "high",
                    "process_after": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "purpose": "purpose",
                    "receiving_account": {
                        "account_details": [
                            {
                                "account_number": "account_number",
                                "account_number_type": "au_number",
                            }
                        ],
                        "account_type": "base_wallet",
                        "contact_details": [
                            {
                                "contact_identifier": "contact_identifier",
                                "contact_identifier_type": "email",
                            }
                        ],
                        "ledger_account": {
                            "currency": "currency",
                            "ledger_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "name": "name",
                            "normal_balance": "credit",
                            "currency_exponent": 0,
                            "description": "description",
                            "ledger_account_category_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                            "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "ledgerable_type": "counterparty",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                        },
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                        "name": "name",
                        "party_address": {
                            "country": "country",
                            "line1": "line1",
                            "line2": "line2",
                            "locality": "locality",
                            "postal_code": "postal_code",
                            "region": "region",
                        },
                        "party_identifier": "party_identifier",
                        "party_name": "party_name",
                        "party_type": "business",
                        "plaid_processor_token": "plaid_processor_token",
                        "routing_details": [
                            {
                                "routing_number": "routing_number",
                                "routing_number_type": "aba",
                                "payment_type": "ach",
                            }
                        ],
                    },
                    "receiving_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "remittance_information": "remittance_information",
                    "send_remittance_advice": True,
                    "statement_descriptor": "statement_descriptor",
                    "subtype": "0C",
                    "transaction_monitoring_enabled": True,
                    "ultimate_originating_party_identifier": "ultimate_originating_party_identifier",
                    "ultimate_originating_party_name": "ultimate_originating_party_name",
                    "ultimate_receiving_party_identifier": "ultimate_receiving_party_identifier",
                    "ultimate_receiving_party_name": "ultimate_receiving_party_name",
                }
            ],
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
        )
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.bulk_requests.with_raw_response.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk_request = response.parse()
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.bulk_requests.with_streaming_response.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk_request = response.parse()
            assert_matches_type(BulkRequest, bulk_request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        bulk_request = client.bulk_requests.retrieve(
            "id",
        )
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.bulk_requests.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk_request = response.parse()
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.bulk_requests.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk_request = response.parse()
            assert_matches_type(BulkRequest, bulk_request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.bulk_requests.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        bulk_request = client.bulk_requests.list()
        assert_matches_type(SyncPage[BulkRequest], bulk_request, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        bulk_request = client.bulk_requests.list(
            action_type="create",
            after_cursor="after_cursor",
            metadata={"foo": "string"},
            per_page=0,
            resource_type="payment_order",
            status="pending",
        )
        assert_matches_type(SyncPage[BulkRequest], bulk_request, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.bulk_requests.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk_request = response.parse()
        assert_matches_type(SyncPage[BulkRequest], bulk_request, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.bulk_requests.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk_request = response.parse()
            assert_matches_type(SyncPage[BulkRequest], bulk_request, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncBulkRequests:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        bulk_request = await async_client.bulk_requests.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                }
            ],
        )
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        bulk_request = await async_client.bulk_requests.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                    "accounting": {
                        "account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "class_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    },
                    "accounting_category_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "accounting_ledger_class_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "charge_bearer": "shared",
                    "currency": "AED",
                    "description": "description",
                    "effective_date": parse_date("2019-12-27"),
                    "expires_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "fallback_type": "ach",
                    "foreign_exchange_contract": "foreign_exchange_contract",
                    "foreign_exchange_indicator": "fixed_to_variable",
                    "ledger_transaction": {
                        "ledger_entries": [
                            {
                                "amount": 0,
                                "direction": "credit",
                                "ledger_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                                "available_balance_amount": {"foo": 0},
                                "lock_version": 0,
                                "metadata": {
                                    "key": "value",
                                    "foo": "bar",
                                    "modern": "treasury",
                                },
                                "pending_balance_amount": {"foo": 0},
                                "posted_balance_amount": {"foo": 0},
                                "show_resulting_ledger_account_balances": True,
                            }
                        ],
                        "description": "description",
                        "effective_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "effective_date": parse_date("2019-12-27"),
                        "external_id": "external_id",
                        "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                        "ledgerable_type": "expected_payment",
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                        "status": "archived",
                    },
                    "ledger_transaction_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "line_items": [
                        {
                            "amount": 0,
                            "accounting_category_id": "accounting_category_id",
                            "description": "description",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                        }
                    ],
                    "metadata": {
                        "key": "value",
                        "foo": "bar",
                        "modern": "treasury",
                    },
                    "nsf_protected": True,
                    "originating_party_name": "originating_party_name",
                    "priority": "high",
                    "process_after": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "purpose": "purpose",
                    "receiving_account": {
                        "account_details": [
                            {
                                "account_number": "account_number",
                                "account_number_type": "au_number",
                            }
                        ],
                        "account_type": "base_wallet",
                        "contact_details": [
                            {
                                "contact_identifier": "contact_identifier",
                                "contact_identifier_type": "email",
                            }
                        ],
                        "ledger_account": {
                            "currency": "currency",
                            "ledger_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "name": "name",
                            "normal_balance": "credit",
                            "currency_exponent": 0,
                            "description": "description",
                            "ledger_account_category_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                            "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                            "ledgerable_type": "counterparty",
                            "metadata": {
                                "key": "value",
                                "foo": "bar",
                                "modern": "treasury",
                            },
                        },
                        "metadata": {
                            "key": "value",
                            "foo": "bar",
                            "modern": "treasury",
                        },
                        "name": "name",
                        "party_address": {
                            "country": "country",
                            "line1": "line1",
                            "line2": "line2",
                            "locality": "locality",
                            "postal_code": "postal_code",
                            "region": "region",
                        },
                        "party_identifier": "party_identifier",
                        "party_name": "party_name",
                        "party_type": "business",
                        "plaid_processor_token": "plaid_processor_token",
                        "routing_details": [
                            {
                                "routing_number": "routing_number",
                                "routing_number_type": "aba",
                                "payment_type": "ach",
                            }
                        ],
                    },
                    "receiving_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "remittance_information": "remittance_information",
                    "send_remittance_advice": True,
                    "statement_descriptor": "statement_descriptor",
                    "subtype": "0C",
                    "transaction_monitoring_enabled": True,
                    "ultimate_originating_party_identifier": "ultimate_originating_party_identifier",
                    "ultimate_originating_party_name": "ultimate_originating_party_name",
                    "ultimate_receiving_party_identifier": "ultimate_receiving_party_identifier",
                    "ultimate_receiving_party_name": "ultimate_receiving_party_name",
                }
            ],
            metadata={
                "key": "value",
                "foo": "bar",
                "modern": "treasury",
            },
        )
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.bulk_requests.with_raw_response.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk_request = response.parse()
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @pytest.mark.skip(reason="Multipart documents aren't constructed properly yet")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.bulk_requests.with_streaming_response.create(
            action_type="create",
            resource_type="payment_order",
            resources=[
                {
                    "amount": 0,
                    "direction": "credit",
                    "originating_account_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                    "type": "ach",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk_request = await response.parse()
            assert_matches_type(BulkRequest, bulk_request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        bulk_request = await async_client.bulk_requests.retrieve(
            "id",
        )
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.bulk_requests.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk_request = response.parse()
        assert_matches_type(BulkRequest, bulk_request, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.bulk_requests.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk_request = await response.parse()
            assert_matches_type(BulkRequest, bulk_request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.bulk_requests.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        bulk_request = await async_client.bulk_requests.list()
        assert_matches_type(AsyncPage[BulkRequest], bulk_request, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        bulk_request = await async_client.bulk_requests.list(
            action_type="create",
            after_cursor="after_cursor",
            metadata={"foo": "string"},
            per_page=0,
            resource_type="payment_order",
            status="pending",
        )
        assert_matches_type(AsyncPage[BulkRequest], bulk_request, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.bulk_requests.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        bulk_request = response.parse()
        assert_matches_type(AsyncPage[BulkRequest], bulk_request, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.bulk_requests.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            bulk_request = await response.parse()
            assert_matches_type(AsyncPage[BulkRequest], bulk_request, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["PaymentActionCreateResponse"]


class PaymentActionCreateResponse(BaseModel):
    id: str

    actionable_id: Optional[str] = None
    """The ID of the associated actionable object."""

    actionable_type: Optional[str] = None
    """The type of the associated actionable object.

    One of `payment_order`, `expected_payment`.
    """

    created_at: datetime

    details: object
    """The specifc details of the payment action based on type."""

    internal_account_id: str
    """The ID of the internal account associated with the payment action."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    status: str
    """The current status of the payment action.

    One of `pending`, `processing`, `sent`, `cancelled`, or `failed`.
    """

    type: str
    """The type of the payment action. Determines the action to be taken."""

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import date, datetime
from typing_extensions import Literal

import httpx

from ... import _legacy_response
from ...types import (
    ledger_transaction_list_params,
    ledger_transaction_create_params,
    ledger_transaction_update_params,
    ledger_transaction_create_reversal_params,
    ledger_transaction_create_partial_post_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from .versions import (
    Versions,
    AsyncVersions,
    VersionsWithRawResponse,
    AsyncVersionsWithRawResponse,
    VersionsWithStreamingResponse,
    AsyncVersionsWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ...pagination import SyncPage, AsyncPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.ledger_transaction import LedgerTransaction

__all__ = ["LedgerTransactions", "AsyncLedgerTransactions"]


class LedgerTransactions(SyncAPIResource):
    @cached_property
    def versions(self) -> Versions:
        return Versions(self._client)

    @cached_property
    def with_raw_response(self) -> LedgerTransactionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return LedgerTransactionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LedgerTransactionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return LedgerTransactionsWithStreamingResponse(self)

    def create(
        self,
        *,
        ledger_entries: Iterable[ledger_transaction_create_params.LedgerEntry],
        description: Optional[str] | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        effective_date: Union[str, date] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        status: Literal["archived", "pending", "posted"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Create a ledger transaction.

        Args:
          ledger_entries: An array of ledger entry objects.

          description: An optional description for internal use.

          effective_at: The timestamp (ISO8601 format) at which the ledger transaction happened for
              reporting purposes.

          effective_date: The date (YYYY-MM-DD) on which the ledger transaction happened for reporting
              purposes.

          external_id: A unique string to represent the ledger transaction. Only one pending or posted
              ledger transaction may have this ID in the ledger.

          ledgerable_id: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the id will be populated here, otherwise null.

          ledgerable_type: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the type will be populated here, otherwise null. This can be one of
              payment_order, incoming_payment_detail, expected_payment, return, paper_item, or
              reversal.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          status: To post a ledger transaction at creation, use `posted`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/ledger_transactions",
            body=maybe_transform(
                {
                    "ledger_entries": ledger_entries,
                    "description": description,
                    "effective_at": effective_at,
                    "effective_date": effective_date,
                    "external_id": external_id,
                    "ledgerable_id": ledgerable_id,
                    "ledgerable_type": ledgerable_type,
                    "metadata": metadata,
                    "status": status,
                },
                ledger_transaction_create_params.LedgerTransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LedgerTransaction:
        """
        Get details on a single ledger transaction.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/ledger_transactions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerTransaction,
        )

    def update(
        self,
        id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        ledger_entries: Iterable[ledger_transaction_update_params.LedgerEntry] | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        status: Literal["archived", "pending", "posted"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Update the details of a ledger transaction.

        Args:
          description: An optional description for internal use.

          effective_at: The timestamp (ISO8601 format) at which the ledger transaction happened for
              reporting purposes.

          ledger_entries: An array of ledger entry objects.

          ledgerable_id: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the id will be populated here, otherwise null.

          ledgerable_type: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the type will be populated here, otherwise null. This can be one of
              payment_order, incoming_payment_detail, expected_payment, return, paper_item, or
              reversal.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          status: To post a ledger transaction at creation, use `posted`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/ledger_transactions/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "effective_at": effective_at,
                    "ledger_entries": ledger_entries,
                    "ledgerable_id": ledgerable_id,
                    "ledgerable_type": ledgerable_type,
                    "metadata": metadata,
                    "status": status,
                },
                ledger_transaction_update_params.LedgerTransactionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )

    def list(
        self,
        *,
        id: List[str] | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        effective_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        effective_date: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        ledger_account_category_id: str | NotGiven = NOT_GIVEN,
        ledger_account_id: str | NotGiven = NOT_GIVEN,
        ledger_account_settlement_id: str | NotGiven = NOT_GIVEN,
        ledger_id: str | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        order_by: ledger_transaction_list_params.OrderBy | NotGiven = NOT_GIVEN,
        partially_posts_ledger_transaction_id: str | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        posted_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        reverses_ledger_transaction_id: str | NotGiven = NOT_GIVEN,
        status: Literal["pending", "posted", "archived"] | NotGiven = NOT_GIVEN,
        updated_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[LedgerTransaction]:
        """
        Get a list of ledger transactions.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          effective_at: Use "gt" (>), "gte" (>=), "lt" (<), "lte" (<=), or "eq" (=) to filter by
              effective at. For example, for all transactions after Jan 1 2000, use
              effective_at%5Bgt%5D=2000-01-01T00:00:00:00.000Z.

          effective_date: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by
              effective date. For example, for all dates after Jan 1 2000, use
              effective_date%5Bgt%5D=2000-01-01.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          order_by: Order by `created_at` or `effective_at` in `asc` or `desc` order. For example,
              to order by `effective_at asc`, use `order_by%5Beffective_at%5D=asc`. Ordering
              by only one field at a time is supported.

          posted_at: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
              posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
              posted_at%5Bgt%5D=2000-01-01T12:00:00Z.

          updated_at: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
              posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
              updated_at%5Bgt%5D=2000-01-01T12:00:00Z.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledger_transactions",
            page=SyncPage[LedgerTransaction],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "effective_at": effective_at,
                        "effective_date": effective_date,
                        "external_id": external_id,
                        "ledger_account_category_id": ledger_account_category_id,
                        "ledger_account_id": ledger_account_id,
                        "ledger_account_settlement_id": ledger_account_settlement_id,
                        "ledger_id": ledger_id,
                        "ledgerable_id": ledgerable_id,
                        "ledgerable_type": ledgerable_type,
                        "metadata": metadata,
                        "order_by": order_by,
                        "partially_posts_ledger_transaction_id": partially_posts_ledger_transaction_id,
                        "per_page": per_page,
                        "posted_at": posted_at,
                        "reverses_ledger_transaction_id": reverses_ledger_transaction_id,
                        "status": status,
                        "updated_at": updated_at,
                    },
                    ledger_transaction_list_params.LedgerTransactionListParams,
                ),
            ),
            model=LedgerTransaction,
        )

    def create_partial_post(
        self,
        id: str,
        *,
        posted_ledger_entries: Iterable[ledger_transaction_create_partial_post_params.PostedLedgerEntry],
        description: str | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Create a ledger transaction that partially posts another ledger transaction.

        Args:
          posted_ledger_entries: An array of ledger entry objects to be set on the posted ledger transaction.
              There must be one entry for each of the existing entries with a lesser amount
              than the existing entry.

          description: An optional free-form description for the posted ledger transaction. Maximum of
              1000 characters allowed.

          effective_at: The timestamp (IS08601 format) at which the posted ledger transaction happened
              for reporting purposes.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/ledger_transactions/{id}/partial_post",
            body=maybe_transform(
                {
                    "posted_ledger_entries": posted_ledger_entries,
                    "description": description,
                    "effective_at": effective_at,
                    "metadata": metadata,
                },
                ledger_transaction_create_partial_post_params.LedgerTransactionCreatePartialPostParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )

    def create_reversal(
        self,
        id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        status: Literal["archived", "pending", "posted"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Create a ledger transaction reversal.

        Args:
          description: An optional free-form description for the reversal ledger transaction. Maximum
              of 1000 characters allowed.

          effective_at: The timestamp (ISO8601 format) at which the reversal ledger transaction happened
              for reporting purposes. It defaults to the `effective_at` of the original ledger
              transaction if not provided.

          external_id: Must be unique within the ledger.

          ledgerable_id: Specify this if you'd like to link the reversal ledger transaction to a Payment
              object like Return or Reversal.

          ledgerable_type: Specify this if you'd like to link the reversal ledger transaction to a Payment
              object like Return or Reversal.

          metadata: Additional data to be added to the reversal ledger transaction as key-value
              pairs. Both the key and value must be strings.

          status: Status of the reversal ledger transaction. It defaults to `posted` if not
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/ledger_transactions/{id}/reversal",
            body=maybe_transform(
                {
                    "description": description,
                    "effective_at": effective_at,
                    "external_id": external_id,
                    "ledgerable_id": ledgerable_id,
                    "ledgerable_type": ledgerable_type,
                    "metadata": metadata,
                    "status": status,
                },
                ledger_transaction_create_reversal_params.LedgerTransactionCreateReversalParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )


class AsyncLedgerTransactions(AsyncAPIResource):
    @cached_property
    def versions(self) -> AsyncVersions:
        return AsyncVersions(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLedgerTransactionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLedgerTransactionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLedgerTransactionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncLedgerTransactionsWithStreamingResponse(self)

    async def create(
        self,
        *,
        ledger_entries: Iterable[ledger_transaction_create_params.LedgerEntry],
        description: Optional[str] | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        effective_date: Union[str, date] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        status: Literal["archived", "pending", "posted"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Create a ledger transaction.

        Args:
          ledger_entries: An array of ledger entry objects.

          description: An optional description for internal use.

          effective_at: The timestamp (ISO8601 format) at which the ledger transaction happened for
              reporting purposes.

          effective_date: The date (YYYY-MM-DD) on which the ledger transaction happened for reporting
              purposes.

          external_id: A unique string to represent the ledger transaction. Only one pending or posted
              ledger transaction may have this ID in the ledger.

          ledgerable_id: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the id will be populated here, otherwise null.

          ledgerable_type: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the type will be populated here, otherwise null. This can be one of
              payment_order, incoming_payment_detail, expected_payment, return, paper_item, or
              reversal.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          status: To post a ledger transaction at creation, use `posted`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/ledger_transactions",
            body=await async_maybe_transform(
                {
                    "ledger_entries": ledger_entries,
                    "description": description,
                    "effective_at": effective_at,
                    "effective_date": effective_date,
                    "external_id": external_id,
                    "ledgerable_id": ledgerable_id,
                    "ledgerable_type": ledgerable_type,
                    "metadata": metadata,
                    "status": status,
                },
                ledger_transaction_create_params.LedgerTransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LedgerTransaction:
        """
        Get details on a single ledger transaction.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/ledger_transactions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LedgerTransaction,
        )

    async def update(
        self,
        id: str,
        *,
        description: Optional[str] | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        ledger_entries: Iterable[ledger_transaction_update_params.LedgerEntry] | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        status: Literal["archived", "pending", "posted"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Update the details of a ledger transaction.

        Args:
          description: An optional description for internal use.

          effective_at: The timestamp (ISO8601 format) at which the ledger transaction happened for
              reporting purposes.

          ledger_entries: An array of ledger entry objects.

          ledgerable_id: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the id will be populated here, otherwise null.

          ledgerable_type: If the ledger transaction can be reconciled to another object in Modern
              Treasury, the type will be populated here, otherwise null. This can be one of
              payment_order, incoming_payment_detail, expected_payment, return, paper_item, or
              reversal.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          status: To post a ledger transaction at creation, use `posted`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/ledger_transactions/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "effective_at": effective_at,
                    "ledger_entries": ledger_entries,
                    "ledgerable_id": ledgerable_id,
                    "ledgerable_type": ledgerable_type,
                    "metadata": metadata,
                    "status": status,
                },
                ledger_transaction_update_params.LedgerTransactionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )

    def list(
        self,
        *,
        id: List[str] | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        effective_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        effective_date: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        ledger_account_category_id: str | NotGiven = NOT_GIVEN,
        ledger_account_id: str | NotGiven = NOT_GIVEN,
        ledger_account_settlement_id: str | NotGiven = NOT_GIVEN,
        ledger_id: str | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        order_by: ledger_transaction_list_params.OrderBy | NotGiven = NOT_GIVEN,
        partially_posts_ledger_transaction_id: str | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        posted_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        reverses_ledger_transaction_id: str | NotGiven = NOT_GIVEN,
        status: Literal["pending", "posted", "archived"] | NotGiven = NOT_GIVEN,
        updated_at: Dict[str, Union[str, datetime]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[LedgerTransaction, AsyncPage[LedgerTransaction]]:
        """
        Get a list of ledger transactions.

        Args:
          id: If you have specific IDs to retrieve in bulk, you can pass them as query
              parameters delimited with `id[]=`, for example `?id[]=123&id[]=abc`.

          effective_at: Use "gt" (>), "gte" (>=), "lt" (<), "lte" (<=), or "eq" (=) to filter by
              effective at. For example, for all transactions after Jan 1 2000, use
              effective_at%5Bgt%5D=2000-01-01T00:00:00:00.000Z.

          effective_date: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by
              effective date. For example, for all dates after Jan 1 2000, use
              effective_date%5Bgt%5D=2000-01-01.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          order_by: Order by `created_at` or `effective_at` in `asc` or `desc` order. For example,
              to order by `effective_at asc`, use `order_by%5Beffective_at%5D=asc`. Ordering
              by only one field at a time is supported.

          posted_at: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
              posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
              posted_at%5Bgt%5D=2000-01-01T12:00:00Z.

          updated_at: Use `gt` (>), `gte` (>=), `lt` (<), `lte` (<=), or `eq` (=) to filter by the
              posted at timestamp. For example, for all times after Jan 1 2000 12:00 UTC, use
              updated_at%5Bgt%5D=2000-01-01T12:00:00Z.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/ledger_transactions",
            page=AsyncPage[LedgerTransaction],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after_cursor": after_cursor,
                        "effective_at": effective_at,
                        "effective_date": effective_date,
                        "external_id": external_id,
                        "ledger_account_category_id": ledger_account_category_id,
                        "ledger_account_id": ledger_account_id,
                        "ledger_account_settlement_id": ledger_account_settlement_id,
                        "ledger_id": ledger_id,
                        "ledgerable_id": ledgerable_id,
                        "ledgerable_type": ledgerable_type,
                        "metadata": metadata,
                        "order_by": order_by,
                        "partially_posts_ledger_transaction_id": partially_posts_ledger_transaction_id,
                        "per_page": per_page,
                        "posted_at": posted_at,
                        "reverses_ledger_transaction_id": reverses_ledger_transaction_id,
                        "status": status,
                        "updated_at": updated_at,
                    },
                    ledger_transaction_list_params.LedgerTransactionListParams,
                ),
            ),
            model=LedgerTransaction,
        )

    async def create_partial_post(
        self,
        id: str,
        *,
        posted_ledger_entries: Iterable[ledger_transaction_create_partial_post_params.PostedLedgerEntry],
        description: str | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Create a ledger transaction that partially posts another ledger transaction.

        Args:
          posted_ledger_entries: An array of ledger entry objects to be set on the posted ledger transaction.
              There must be one entry for each of the existing entries with a lesser amount
              than the existing entry.

          description: An optional free-form description for the posted ledger transaction. Maximum of
              1000 characters allowed.

          effective_at: The timestamp (IS08601 format) at which the posted ledger transaction happened
              for reporting purposes.

          metadata: Additional data represented as key-value pairs. Both the key and value must be
              strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/ledger_transactions/{id}/partial_post",
            body=await async_maybe_transform(
                {
                    "posted_ledger_entries": posted_ledger_entries,
                    "description": description,
                    "effective_at": effective_at,
                    "metadata": metadata,
                },
                ledger_transaction_create_partial_post_params.LedgerTransactionCreatePartialPostParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )

    async def create_reversal(
        self,
        id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        effective_at: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        ledgerable_id: str | NotGiven = NOT_GIVEN,
        ledgerable_type: Literal[
            "expected_payment", "incoming_payment_detail", "paper_item", "payment_order", "return", "reversal"
        ]
        | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        status: Literal["archived", "pending", "posted"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> LedgerTransaction:
        """
        Create a ledger transaction reversal.

        Args:
          description: An optional free-form description for the reversal ledger transaction. Maximum
              of 1000 characters allowed.

          effective_at: The timestamp (ISO8601 format) at which the reversal ledger transaction happened
              for reporting purposes. It defaults to the `effective_at` of the original ledger
              transaction if not provided.

          external_id: Must be unique within the ledger.

          ledgerable_id: Specify this if you'd like to link the reversal ledger transaction to a Payment
              object like Return or Reversal.

          ledgerable_type: Specify this if you'd like to link the reversal ledger transaction to a Payment
              object like Return or Reversal.

          metadata: Additional data to be added to the reversal ledger transaction as key-value
              pairs. Both the key and value must be strings.

          status: Status of the reversal ledger transaction. It defaults to `posted` if not
              provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/ledger_transactions/{id}/reversal",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "effective_at": effective_at,
                    "external_id": external_id,
                    "ledgerable_id": ledgerable_id,
                    "ledgerable_type": ledgerable_type,
                    "metadata": metadata,
                    "status": status,
                },
                ledger_transaction_create_reversal_params.LedgerTransactionCreateReversalParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=LedgerTransaction,
        )


class LedgerTransactionsWithRawResponse:
    def __init__(self, ledger_transactions: LedgerTransactions) -> None:
        self._ledger_transactions = ledger_transactions

        self.create = _legacy_response.to_raw_response_wrapper(
            ledger_transactions.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            ledger_transactions.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            ledger_transactions.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            ledger_transactions.list,
        )
        self.create_partial_post = _legacy_response.to_raw_response_wrapper(
            ledger_transactions.create_partial_post,
        )
        self.create_reversal = _legacy_response.to_raw_response_wrapper(
            ledger_transactions.create_reversal,
        )

    @cached_property
    def versions(self) -> VersionsWithRawResponse:
        return VersionsWithRawResponse(self._ledger_transactions.versions)


class AsyncLedgerTransactionsWithRawResponse:
    def __init__(self, ledger_transactions: AsyncLedgerTransactions) -> None:
        self._ledger_transactions = ledger_transactions

        self.create = _legacy_response.async_to_raw_response_wrapper(
            ledger_transactions.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            ledger_transactions.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            ledger_transactions.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            ledger_transactions.list,
        )
        self.create_partial_post = _legacy_response.async_to_raw_response_wrapper(
            ledger_transactions.create_partial_post,
        )
        self.create_reversal = _legacy_response.async_to_raw_response_wrapper(
            ledger_transactions.create_reversal,
        )

    @cached_property
    def versions(self) -> AsyncVersionsWithRawResponse:
        return AsyncVersionsWithRawResponse(self._ledger_transactions.versions)


class LedgerTransactionsWithStreamingResponse:
    def __init__(self, ledger_transactions: LedgerTransactions) -> None:
        self._ledger_transactions = ledger_transactions

        self.create = to_streamed_response_wrapper(
            ledger_transactions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            ledger_transactions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            ledger_transactions.update,
        )
        self.list = to_streamed_response_wrapper(
            ledger_transactions.list,
        )
        self.create_partial_post = to_streamed_response_wrapper(
            ledger_transactions.create_partial_post,
        )
        self.create_reversal = to_streamed_response_wrapper(
            ledger_transactions.create_reversal,
        )

    @cached_property
    def versions(self) -> VersionsWithStreamingResponse:
        return VersionsWithStreamingResponse(self._ledger_transactions.versions)


class AsyncLedgerTransactionsWithStreamingResponse:
    def __init__(self, ledger_transactions: AsyncLedgerTransactions) -> None:
        self._ledger_transactions = ledger_transactions

        self.create = async_to_streamed_response_wrapper(
            ledger_transactions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            ledger_transactions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            ledger_transactions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ledger_transactions.list,
        )
        self.create_partial_post = async_to_streamed_response_wrapper(
            ledger_transactions.create_partial_post,
        )
        self.create_reversal = async_to_streamed_response_wrapper(
            ledger_transactions.create_reversal,
        )

    @cached_property
    def versions(self) -> AsyncVersionsWithStreamingResponse:
        return AsyncVersionsWithStreamingResponse(self._ledger_transactions.versions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .account_entries import (
    AccountEntries,
    AsyncAccountEntries,
    AccountEntriesWithRawResponse,
    AsyncAccountEntriesWithRawResponse,
    AccountEntriesWithStreamingResponse,
    AsyncAccountEntriesWithStreamingResponse,
)
from .ledger_account_settlements import (
    LedgerAccountSettlements,
    AsyncLedgerAccountSettlements,
    LedgerAccountSettlementsWithRawResponse,
    AsyncLedgerAccountSettlementsWithRawResponse,
    LedgerAccountSettlementsWithStreamingResponse,
    AsyncLedgerAccountSettlementsWithStreamingResponse,
)

__all__ = [
    "AccountEntries",
    "AsyncAccountEntries",
    "AccountEntriesWithRawResponse",
    "AsyncAccountEntriesWithRawResponse",
    "AccountEntriesWithStreamingResponse",
    "AsyncAccountEntriesWithStreamingResponse",
    "LedgerAccountSettlements",
    "AsyncLedgerAccountSettlements",
    "LedgerAccountSettlementsWithRawResponse",
    "AsyncLedgerAccountSettlementsWithRawResponse",
    "LedgerAccountSettlementsWithStreamingResponse",
    "AsyncLedgerAccountSettlementsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import payment_action_list_params, payment_action_create_params, payment_action_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.payment_action_list_response import PaymentActionListResponse
from ..types.payment_action_create_response import PaymentActionCreateResponse
from ..types.payment_action_update_response import PaymentActionUpdateResponse
from ..types.payment_action_retrieve_response import PaymentActionRetrieveResponse

__all__ = ["PaymentActions", "AsyncPaymentActions"]


class PaymentActions(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PaymentActionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return PaymentActionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PaymentActionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return PaymentActionsWithStreamingResponse(self)

    def create(
        self,
        *,
        type: str,
        actionable_id: str | NotGiven = NOT_GIVEN,
        actionable_type: str | NotGiven = NOT_GIVEN,
        details: object | NotGiven = NOT_GIVEN,
        internal_account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PaymentActionCreateResponse:
        """Create a payment action.

        Args:
          type: Required.

        The type of the payment action. Determines the action to be taken.

          actionable_id: Optional. The ID of the associated actionable object.

          actionable_type: Optional. The type of the associated actionable object. One of `payment_order`,
              `expected_payment`. Required if `actionable_id` is passed.

          details: Optional. The specifc details of the payment action based on type.

          internal_account_id: Optional. The ID of one of your organization's internal accounts. Required if
              `actionable_id` is not passed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/payment_actions",
            body=maybe_transform(
                {
                    "type": type,
                    "actionable_id": actionable_id,
                    "actionable_type": actionable_type,
                    "details": details,
                    "internal_account_id": internal_account_id,
                },
                payment_action_create_params.PaymentActionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PaymentActionCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PaymentActionRetrieveResponse:
        """
        Get details on a single payment action.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/payment_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentActionRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        status: Literal["pending", "processable", "processing", "sent", "failed", "cancelled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PaymentActionUpdateResponse:
        """Update a single payment action.

        Args:
          status: Optional.

        Set the status of the payment action to `cancelled` to cancel the
              payment action. This will only work if the payment action is in a `pending`
              state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/payment_actions/{id}",
            body=maybe_transform({"status": status}, payment_action_update_params.PaymentActionUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PaymentActionUpdateResponse,
        )

    def list(
        self,
        *,
        actionable_id: str | NotGiven = NOT_GIVEN,
        actionable_type: str | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        created_at: payment_action_list_params.CreatedAt | NotGiven = NOT_GIVEN,
        internal_account_id: str | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        status: Literal["pending", "processable", "processing", "sent", "failed", "cancelled"] | NotGiven = NOT_GIVEN,
        type: Literal["stop", "issue"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPage[PaymentActionListResponse]:
        """
        Get a list of all payment actions.

        Args:
          actionable_id: The ID of the associated actionable object.

          actionable_type: The type of the associated actionable object. One of `payment_order`,
              `expected_payment`.

          created_at: Filter by `created_at` using comparison operators like `gt` (>), `gte` (>=),
              `lt` (<), `lte` (<=), or `eq` (=) to filter by the created at timestamp. For
              example, `created_at[gte]=2024-01-01T00:00:00Z`

          internal_account_id: The ID of one of your internal accounts.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          status: Filter by payment actions of a specific status.

          type: The type of payment action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/payment_actions",
            page=SyncPage[PaymentActionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "actionable_id": actionable_id,
                        "actionable_type": actionable_type,
                        "after_cursor": after_cursor,
                        "created_at": created_at,
                        "internal_account_id": internal_account_id,
                        "metadata": metadata,
                        "per_page": per_page,
                        "status": status,
                        "type": type,
                    },
                    payment_action_list_params.PaymentActionListParams,
                ),
            ),
            model=PaymentActionListResponse,
        )


class AsyncPaymentActions(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPaymentActionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPaymentActionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPaymentActionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncPaymentActionsWithStreamingResponse(self)

    async def create(
        self,
        *,
        type: str,
        actionable_id: str | NotGiven = NOT_GIVEN,
        actionable_type: str | NotGiven = NOT_GIVEN,
        details: object | NotGiven = NOT_GIVEN,
        internal_account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PaymentActionCreateResponse:
        """Create a payment action.

        Args:
          type: Required.

        The type of the payment action. Determines the action to be taken.

          actionable_id: Optional. The ID of the associated actionable object.

          actionable_type: Optional. The type of the associated actionable object. One of `payment_order`,
              `expected_payment`. Required if `actionable_id` is passed.

          details: Optional. The specifc details of the payment action based on type.

          internal_account_id: Optional. The ID of one of your organization's internal accounts. Required if
              `actionable_id` is not passed.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/payment_actions",
            body=await async_maybe_transform(
                {
                    "type": type,
                    "actionable_id": actionable_id,
                    "actionable_type": actionable_type,
                    "details": details,
                    "internal_account_id": internal_account_id,
                },
                payment_action_create_params.PaymentActionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PaymentActionCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PaymentActionRetrieveResponse:
        """
        Get details on a single payment action.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/payment_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentActionRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        status: Literal["pending", "processable", "processing", "sent", "failed", "cancelled"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PaymentActionUpdateResponse:
        """Update a single payment action.

        Args:
          status: Optional.

        Set the status of the payment action to `cancelled` to cancel the
              payment action. This will only work if the payment action is in a `pending`
              state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/payment_actions/{id}",
            body=await async_maybe_transform(
                {"status": status}, payment_action_update_params.PaymentActionUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PaymentActionUpdateResponse,
        )

    def list(
        self,
        *,
        actionable_id: str | NotGiven = NOT_GIVEN,
        actionable_type: str | NotGiven = NOT_GIVEN,
        after_cursor: Optional[str] | NotGiven = NOT_GIVEN,
        created_at: payment_action_list_params.CreatedAt | NotGiven = NOT_GIVEN,
        internal_account_id: str | NotGiven = NOT_GIVEN,
        metadata: Dict[str, str] | NotGiven = NOT_GIVEN,
        per_page: int | NotGiven = NOT_GIVEN,
        status: Literal["pending", "processable", "processing", "sent", "failed", "cancelled"] | NotGiven = NOT_GIVEN,
        type: Literal["stop", "issue"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PaymentActionListResponse, AsyncPage[PaymentActionListResponse]]:
        """
        Get a list of all payment actions.

        Args:
          actionable_id: The ID of the associated actionable object.

          actionable_type: The type of the associated actionable object. One of `payment_order`,
              `expected_payment`.

          created_at: Filter by `created_at` using comparison operators like `gt` (>), `gte` (>=),
              `lt` (<), `lte` (<=), or `eq` (=) to filter by the created at timestamp. For
              example, `created_at[gte]=2024-01-01T00:00:00Z`

          internal_account_id: The ID of one of your internal accounts.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          status: Filter by payment actions of a specific status.

          type: The type of payment action.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/payment_actions",
            page=AsyncPage[PaymentActionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "actionable_id": actionable_id,
                        "actionable_type": actionable_type,
                        "after_cursor": after_cursor,
                        "created_at": created_at,
                        "internal_account_id": internal_account_id,
                        "metadata": metadata,
                        "per_page": per_page,
                        "status": status,
                        "type": type,
                    },
                    payment_action_list_params.PaymentActionListParams,
                ),
            ),
            model=PaymentActionListResponse,
        )


class PaymentActionsWithRawResponse:
    def __init__(self, payment_actions: PaymentActions) -> None:
        self._payment_actions = payment_actions

        self.create = _legacy_response.to_raw_response_wrapper(
            payment_actions.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            payment_actions.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            payment_actions.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            payment_actions.list,
        )


class AsyncPaymentActionsWithRawResponse:
    def __init__(self, payment_actions: AsyncPaymentActions) -> None:
        self._payment_actions = payment_actions

        self.create = _legacy_response.async_to_raw_response_wrapper(
            payment_actions.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            payment_actions.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            payment_actions.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            payment_actions.list,
        )


class PaymentActionsWithStreamingResponse:
    def __init__(self, payment_actions: PaymentActions) -> None:
        self._payment_actions = payment_actions

        self.create = to_streamed_response_wrapper(
            payment_actions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            payment_actions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            payment_actions.update,
        )
        self.list = to_streamed_response_wrapper(
            payment_actions.list,
        )


class AsyncPaymentActionsWithStreamingResponse:
    def __init__(self, payment_actions: AsyncPaymentActions) -> None:
        self._payment_actions = payment_actions

        self.create = async_to_streamed_response_wrapper(
            payment_actions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            payment_actions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            payment_actions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            payment_actions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.transaction_direction import TransactionDirection

__all__ = ["LedgerAccount", "Balances", "BalancesAvailableBalance", "BalancesPendingBalance", "BalancesPostedBalance"]


class BalancesAvailableBalance(BaseModel):
    amount: int

    credits: int

    currency: str
    """The currency of the ledger account."""

    currency_exponent: int
    """The currency exponent of the ledger account."""

    debits: int


class BalancesPendingBalance(BaseModel):
    amount: int

    credits: int

    currency: str
    """The currency of the ledger account."""

    currency_exponent: int
    """The currency exponent of the ledger account."""

    debits: int


class BalancesPostedBalance(BaseModel):
    amount: int

    credits: int

    currency: str
    """The currency of the ledger account."""

    currency_exponent: int
    """The currency exponent of the ledger account."""

    debits: int


class Balances(BaseModel):
    available_balance: BalancesAvailableBalance
    """
    The available_balance is the sum of all posted inbound entries and pending
    outbound entries. For credit normal, available_amount = posted_credits -
    pending_debits; for debit normal, available_amount = posted_debits -
    pending_credits.
    """

    effective_at_lower_bound: Optional[datetime] = None
    """
    The inclusive lower bound of the effective_at timestamp for the returned
    balances.
    """

    effective_at_upper_bound: Optional[datetime] = None
    """
    The exclusive upper bound of the effective_at timestamp for the returned
    balances.
    """

    pending_balance: BalancesPendingBalance
    """The pending_balance is the sum of all pending and posted entries."""

    posted_balance: BalancesPostedBalance
    """The posted_balance is the sum of all posted entries."""


class LedgerAccount(BaseModel):
    id: str

    balances: Balances
    """The pending, posted, and available balances for this ledger account.

    The posted balance is the sum of all posted entries on the account. The pending
    balance is the sum of all pending and posted entries on the account. The
    available balance is the posted incoming entries minus the sum of the pending
    and posted outgoing amounts.
    """

    created_at: datetime

    description: Optional[str] = None
    """The description of the ledger account."""

    discarded_at: Optional[datetime] = None

    external_id: Optional[str] = None
    """An optional user-defined 180 character unique identifier."""

    ledger_id: str
    """The id of the ledger that this account belongs to."""

    ledgerable_id: Optional[str] = None
    """
    If the ledger account links to another object in Modern Treasury, the id will be
    populated here, otherwise null.
    """

    ledgerable_type: Optional[Literal["counterparty", "external_account", "internal_account", "virtual_account"]] = None
    """
    If the ledger account links to another object in Modern Treasury, the type will
    be populated here, otherwise null. The value is one of internal_account or
    external_account.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    lock_version: int
    """Lock version of the ledger account."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: str
    """The name of the ledger account."""

    normal_balance: TransactionDirection
    """The normal balance of the ledger account."""

    object: str

    updated_at: datetime
